@@ -307,7 +307,8 @@ static size_t write_cb(void *ptr _Unused, size_t size, size_t nmemb,
                        void *data)
 {
   size_t realsize = size * nmemb;
-  ConnInfo *conn _Unused = (ConnInfo*) data;
+  (void)_Unused;
+  (void)data;
 
   return realsize;
 }
